package ALTLinux::RepoList;

use strict;
use warnings;
use Carp;
use Source::Shared::FindLocalMirror::ALTLinux;
use Source::Shared::CLI;
our @ISA = qw/Source::Shared::CLI/;

our $verbose=0;
$verbose=1 if $ENV{TERM}; # not to pollute cron mail

my @binlists;
my @srclists;
my @REPO_ROOTS;

my $infile;

our @LONGOPT=(
    'repo=s' => \@REPO_ROOTS,
    'bin-list=s' => \@binlists,
    'src-list=s' => \@srclists,
    'infile=s' => \$infile,
    );
sub print_longopt {
    print "  ALTLinux::RepoList options:
	--listdir <dir> /path/to/files/list/ dir
	--bin-list /path/to/bin.list
	--src-list /path/to/src.list
	--infile input file (optional)
";
}

sub cli_container_ref {
    return ['Source::Shared::FindLocalMirror::ALTLinux'];
}

our $need_bin=0;
our $need_src=0;
our @IN;

sub process_options {
    if ($need_bin and @binlists==0 or $need_src and @srclists==0) {
	push @REPO_ROOTS, &Source::Shared::FindLocalMirror::ALTLinux::find_cli(-subpath=>'files/list') unless @REPO_ROOTS;
	push @srclists, map {$_.'/src.list'} @REPO_ROOTS;
	push @binlists, map {$_.'/bin.list'} @REPO_ROOTS;
    }

    if ($need_bin and @binlists==0 or $need_src and @srclists==0) {
	warn "Can't find bin.list. Use --bin-list=/path\n" if $need_bin and @binlists==0;
	warn "Can't find src.list. Use --src-list=/path\n" if $need_src and @srclists==0;
	die "or use --listdir <dir> /path/to/files/list/ dir\n";
    }

    # ------- @IN --------------
    &__load_file_lambda2($infile,
    sub {
	my ($fn)=@_;
	while (my $line=<$fn>) {
	    chomp $line;
	    $line=~s/^\s*//;
	    $line=~s/\s*$//;
	    push @IN, $line;
	}
	}) if $infile;
    push @IN, @ARGV;
    # ------- END @IN -----------
}

# public ------------------------------------------------------------

sub process_bin_lists {
    my ($lambda)=@_;
    return map {&__process_bin_list($_,$lambda)} @binlists;
}

sub process_src_lists {
    my ($lambda)=@_;
    return map {&__process_src_list($_,$lambda)} @srclists;
}

# protected ------------------------------------------------------------

sub __process_bin_list {
    my ($list,$lambda)=@_;
    my $root=&__get_repo_root($list);
    return &__load_file_lambda2($list,\&__read_altbinlist_fn_lambda,$lambda,$root);
}


sub __process_src_list {
    my ($list,$lambda)=@_;
    my $root=&__get_repo_root($list);
    return &__load_file_lambda2($list,\&__read_altsrclist_fn_lambda,$lambda,$root);
}

# private ------------------------------------------------------------

sub __load_file_lambda2 {
    my ($list, $readfunc,$linefunc, $reporoot)=@_;
    open (my $fn, '<', $list) or die "can't open file $list\n";
    my $ret=&$readfunc($fn,$linefunc,$reporoot);
    close($fn);
    return $ret;
}

sub __read_altsrclist_fn_lambda {
    my ($fn,$lambda,$reporoot)=@_;
    while (my $line=<$fn>) {
	chomp $line;
	my @list = split(/\s+/,$line);
	next if @list!=3;
	my ($name,$evr,$srpm)=@list;
	&$lambda($name,$evr,$srpm,$reporoot);
    }
}

sub __read_altbinlist_fn_lambda {
    my ($fn,$lambda,$reporoot)=@_;
    while (my $line=<$fn>) {
	chomp $line;
	my @list = split(/\s+/,$line);
	next if @list!=5;
	my ($name,$evr,$arch,$rpm,$srpm)=@list;
	&$lambda($name,$evr,$arch,$rpm,$srpm,$reporoot);
    }
}

sub __get_repo_root {
    my $list=shift;
    if ($list =~ s!/files/list/(?:src|bin).list$!!) {
	return $list;
    } else {
	confess "Internal error: not a list: $list";
    }
}

# ------------------------------------------------------------

1;
